/*
   	Public interfaces to the BT848 driver (device control opcodes, structs, etc).
*/

#ifndef _Bt848_H
#define _Bt848_H

#ifndef _DRIVERS_H
#include <Drivers.h>
#endif

/* -----
	ioctl opcodes
----- */

enum {
	BT848_INIT = B_DEVICE_OP_CODES_END + 1,
	BT848_START,
	BT848_STOP,
	BT848_STATUS,
	BT848_BRIGHT,
	BT848_CONTRAST,
	BT848_HUE,
	BT848_SATURATION,
	BT848_SET_CHANNEL,
	BT848_SELECT_AUDIO_SOURCE,
	BT848_I2CREAD,
	BT848_I2CWRITE
};

/*
	Bt848 configuration structure and value definitions 
*/

typedef struct {
	int				source;			/* video source 0-3 */
	int				format;			/* video formet (NTSC,PAL, etc--see defines below) */
	int				x_size;			/* scaled video image width in pixels */
	int				y_size;			/* scaled video image height in pixels */
	int				color_format;	/* color format for output pixels (RGB32,RGB24, etc -- see defines below) */
	int				bright;			/* brightness control default: 0, -100 to +100*/
	int				contrast;		/* contrast control default:0, -100 to +100 */
	int				hue;			/* hue adjustment default: 0, -100 to +100 */
	int				saturation;		/* saturation adjustment default: 0, -100 to +100 */
	int				capture_mode;	/* continuous or single frame/field capture */
	void		 	**o_line;		/* pointer to an array of pointers to storage for odd field scan lines */
									/* set this pointer to NULL to disable capture of odd field */
	int				o_address_type;	/* indicates whether the addresses in o_line are logical or physical */
									/* physical addresses are PCI addresses (ie use ramaddress() ) */
	void			**o_clip;		/* pointer to array of pointers to clip list */
									/* clip list is an null terminated arrary of int runs */
									/* positive values for number of pixels to display    */
									/* negative values for number of pixels to skip       */ 
									/* set this pointer to null to disable clipping */
	void		 	**e_line;		/* pointer to an array of pointers to storage for even fieldscan lines */
									/* set this pointer to NULL to disable capture of even field */
	int				e_address_type;	/* logical or physical addresses in e_line */
									/* physical addresses are PCI addresses (ie use ramaddress() ) */
	void			**e_clip;		/* pointer to array of pointers to clip list */
									/* clip list is an null terminated arrary of int runs */
									/* positive values for number of pixels to display    */
									/* negative values for number of pixels to skip       */ 
									/* set this pointer to null to disable clipping */
	int				decimate;		/* number of frames per second to skip */
	int				status;			/* status return variable */
	int				channel;		/* tuner channel desired */
	int				locale;			/* locale for tuner -- see defines below */
	int				audio_source;	/* audio source -- see defines below */
	unsigned long	i2c;			/* I2C Data/Control */
	
} Bt848_config;

/* defines for source */
#define	COMPOSITE	3
#define	TUNER		2
#define	SVIDEO		1			
	
/* defines for format */
#define	NTSC_M		1
#define	NTSC_JAPAN	2
#define	PAL_BDGHI	3
#define PAL_M		4
#define	PAL_N		5
#define SECAM		6

/* defines for color_format */
#define RGB32		0
#define	RGB24		1
#define	RGB16		2
#define RGB15		3
#define Y8			6
#define	RGB8		7

/* defines for capture mode */
#define CONTINUOUS	1
#define SINGLE		2

/* defines for address_type */
#define	LOGICAL			1
#define PHYSICAL		2

/* defines for status */
#define	RACK			0x02000000
#define RISC_IRQ		0x00000800
#define	I2CDONE			0x00000100

/* defines for locale */
#define US_NTSC_CABLE	0
#define US_NTSC_AIR		1

/* defines for audio_source */
#define	TUNER_AUDIO		0
#define CD_JACK_AUDIO	1
#define RCA_JACK_AUDIO	2
#define MUTE_AUDIO		3

#endif  /* _Bt848_H */

