/*
	
	HelloWindow.cpp
	
	Copyright 1997 Be Incorporated, All Rights Reserved.
	
*/

#include <Application.h>
#include "tvwindow.h"


#include <OS.h>
#include "Bt848_driver.h"

extern int fd;
int cur_screen_x,cur_screen_y,cur_size_x,cur_size_y;
int cur_channel = 2;

void playvideo(int screen_x, int screen_y, int size_x, int size_y);

HelloWindow::HelloWindow(BRect frame)
				: BWindow(frame, "Video Player", B_TITLED_WINDOW, B_NOT_ZOOMABLE)
{
	cur_screen_x = 100;
	cur_screen_y = 20;
	cur_size_x 	= 320;
	cur_size_y 	= 240;
	playvideo(cur_screen_x,cur_screen_y,cur_size_x,cur_size_y);
}

bool HelloWindow::QuitRequested()
{
	be_app->PostMessage(B_QUIT_REQUESTED);
	return(TRUE);
}

void HelloWindow::FrameMoved(BPoint a_point)
{
	cur_screen_x = (int)a_point.x;
	cur_screen_y = (int)a_point.y;
	
	playvideo(cur_screen_x,cur_screen_y,cur_size_x,cur_size_y);
}

void HelloWindow::FrameResized(float h, float v)
{
	cur_size_x = (int)h-12;
	cur_size_y = (int)v-30;

	if (cur_size_x > 640)
		cur_size_x = 640;

	if (cur_size_y > 480)
		cur_size_y = 480;
			
	playvideo(cur_screen_x,cur_screen_y,cur_size_x,cur_size_y);

}

void
HelloWindow::MessageReceived(BMessage *msg)
{
	//printf("MessageReceived:\n");
	
	switch (msg->what)
	{
		case B_KEY_DOWN:
		{
			long key = msg->FindLong("key");
			long aChar = msg->FindLong("char");
			switch(aChar)
			{
				case B_ESCAPE:
					be_app->PostMessage(B_QUIT_REQUESTED);
				break;

				case '+':
					cur_channel++;
					
					playvideo(cur_screen_x,cur_screen_y,cur_size_x,cur_size_y);
				break;
		
				case '-':
					cur_channel--;
					if (cur_channel < 2)
						cur_channel = 2;
					playvideo(cur_screen_x,cur_screen_y,cur_size_x,cur_size_y);
				break;
		
			}
		}
	
		default:
			BWindow::MessageReceived(msg);
		break;
	}
	
}

void playvideo(int screen_x, int screen_y, int size_x, int size_y) {
	
	unsigned long i,step,bytes_per_pixel,screen_width;
	void 	*frame_base,*screen_base;
	void	*even[512];
	void	*odd[512];
	void	*e_clip[512];
	void	*o_clip[512];
	
	int		clip1[9];
	int		clip2[9];

	Bt848_config config;

	_kioctl_(fd,BT848_STOP,&config);
	
	config.source = TUNER;	
	//config.source = COMPOSITE;	
	config.format = NTSC_M;
	config.x_size = size_x;
	config.y_size = size_y;
	config.color_format = RGB32;
	config.bright = 0;
	config.contrast = 0;
	config.hue = 0;
	config.saturation = 0;
	config.capture_mode = CONTINUOUS;
	config.e_line = even;
	config.e_address_type = PHYSICAL;
	config.e_clip = 0/*e_clip*/;
	config.o_line = odd;
	config.o_address_type = PHYSICAL;
	config.o_clip = 0/*o_clip*/;
	config.decimate = 0;
	config.status = 0;

	clip1[0] =            -size_x/8;					/* some test clip arrays */
	clip2[0] = clip1[1] =  size_x/8;
	clip2[1] = clip1[2] = -size_x/8;
	clip2[2] = clip1[3] =  size_x/8;
	clip2[3] = clip1[4] = -size_x/8;
	clip2[4] = clip1[5] =  size_x/8;
	clip2[5] = clip1[6] = -size_x/8;
	clip2[6] =             size_x/8;
	clip1[7] =             size_x - 7 * (size_x/8);		/* make sure that clip runs add up */
	clip2[7] =            -(size_x - 7 * (size_x/8));	/* -exactly- to size_x -- the driver */
	clip2[8] = clip1[8] =  0;							/* gets unhappy otherwise */
			
	screen_base = (void *)(0x10000000 + 32);
	screen_width = 800;
	bytes_per_pixel = 4;
	
	frame_base = (void *)((unsigned long)screen_base + 
		screen_y*screen_width*bytes_per_pixel + 
		screen_x*bytes_per_pixel-28 ); /* frame base address from PCI */
	
	if (config.y_size <= 240)  	/* overlay single fields if CIF or lower resolution */
		step = 1;
	else
		step = 2;				/* otherwise interlace */
		
	for (i=0; i < config.y_size/step; i++)
	{
		odd[i] = (void *)((unsigned long)frame_base + ((i*step)*screen_width*bytes_per_pixel));
		if ( (((i/32) * 32) == i) |
			 ((((i+1)/32) * 32) == (i+1)) |
			 ((((i+2)/32) * 32) == (i+2)))
			o_clip[i] = clip1;
		else
			o_clip[i] = clip2;
	}

	for (i=0; i < config.y_size/step; i++)
	{
		even[i] = (void *)((unsigned long)frame_base + (((i*step)+(step-1))*screen_width*bytes_per_pixel));
		if ( (((i/32) * 32) == i) |
			 ((((i+1)/32) * 32) == (i+1)) |
			 ((((i+2)/32) * 32) == (i+2)))
			e_clip[i] = clip1;
		else
			e_clip[i] = clip2;
	}
	
	/* Initialize Bt848 and start video capture */
	_kioctl_(fd,BT848_INIT,&config);
	_kioctl_(fd,BT848_START,&config);

	/* tune to channel 2 */
	config.channel = cur_channel;
	config.locale = US_NTSC_AIR;
	//config.locale = US_NTSC_CABLE;
	printf("channel: %d\n", cur_channel);
	_kioctl_(fd,BT848_SET_CHANNEL,&config);

	/* select tuner audio */
	config.audio_source = TUNER_AUDIO;
	_kioctl_(fd,BT848_SELECT_AUDIO_SOURCE,&config);
	
	/* uncomment the line below to synchronize single frame or single field capture modes */
	//_kread_(fd,&dummy,1);	


}
